@@ -666,7 +666,7 @@ def _defineclasses(streamerinfos, classes):
 
         if isinstance(streamerinfo, TStreamerInfo) and pyclassname not in builtin_classes and (pyclassname not in classes or hasattr(classes[pyclassname], "_versions")):
             code = ["    @classmethod",
-                    "    def _readinto(cls, self, source, cursor, context, parent):",
+                    "    def _readinto(cls, self, source, cursor, context, parent, asclass=None):",
                     "        start, cnt, classversion = _startcheck(source, cursor)",
                     "        if cls._classversion != classversion:",
                     "            cursor.index = start",
